// dialogs.h
// routines for most dialogs
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 
//          by:

#ifndef DIALOGS_H
#define DIALOGS_H

#include "classes.h"
#include "slideshow.h"
#include "SettingsDialog.h"
#include "client.h"
#include "storage.h"
#include "invitation.h"
#include <qlistbox.h>

class Dialogs;
class UserObservationWindow;


class Dialogs : public QWidget
{
	Q_OBJECT
public:
	Dialogs(ClientStorage *theDataPointer, QWidget *parent=0, const char *name=0);
	QMessageBox		*About;
	SettingsDialog  *Settings;
	QDialog			*UserObservation;
	QListView		*SessionEventsLV;
	QListView		*AllEventsLV;
	QDialog			*Clientwindow; 
	QDialog			*InvitationsWindow; 
	
	bool openClientForUploadAndIfSuccessStoreEverything();
	void startSettingsDialog(bool startWithAlertBox, bool firstTime);
	void createUserObservationWindow(Usage & Observation);
	void updateEventsViewerWindow(Usage & Observation);
	void appendToArchiveFile(ClientStorage theData, QString filename);
	void goodbyeShow(SlideShow *SlideShowPointer);
	void TableOfErrorsAndRecommendations(QWidget *parent);
		// for database:
	static char* giveAllMemberCountriesAsTextTupels(char* thirdcolumn);
	static char* giveAllCitySizesAsTextTupels(char* thirdcolumn);

	void setPicsAlsoInInvitations(QPixmap pics[NumCoins][NumCountries+1], QSize theMaxSize);
	void CopyFromThenExecInvitationWindowAndCopyTo(QValueList<InvitedPerson > &lip);

private:
	Client          *theClient ;   
	Invitations		*theInvitations;
	QDialog			*byebye;
	QLabel			*byepic;
	char			byecoin;
	char			byecountry;
	SlideShow		*ss; // Pointer to SlideShow to have access to the pics
	int	 SettingsValidationOK(bool firstTime);
	void SettingsCopyToStorage(ClientStorage & S);
	void SettingsCopyFromStorage(const ClientStorage & D);
	// to enter several recommender usernumbers:
	QDialog		*severalRecommendersDialog;
	QLineEdit	*newRecommender;
	QListBox	*recommendersLB;
	QLabel		*errorLabel;
	bool commaSeparatedToListBox(QString str, int &errorpos, QListBox * LB);
	QString ListBoxToCommaSeparated(QListBox * LB);
	bool    ListBoxToValueList(QListBox *LB, QValueList<Q_UINT32> &intoTheList);
	void ValueListToListBox(QValueList<Q_UINT32> theList, QListBox *LB);

	// user observation events:
	void SettingsCreateObservationEvents();

public slots:
	void SettingsResetToOldValues();

private slots:
	void createSeveralRecommenderNumbersDialog();
	void execSeveralRecommenderNumbersDialog();
	void addNewRecommender();
	void delRecommender();
	void SettingsCountryChanged(int index);
	void MixdateYearChanged(int dummy = 0);
	void MixdateMonthChanged(int dummy=0);
	void ArrivaldateYearChanged(int dummy = 0);
	void ArrivaldateMonthChanged(int dummy =0);
	// the end show:
	void quit_flipToNextPic();
	// Events for user observation:
	void eventCountry(int dummy);
	void eventPostalCode(const QString&);
	void eventPopulation(int dummy );
	void eventParagraph1(bool dummy);
	void eventParagraph2(bool dummy);
	void eventRecommenderMembernumber(const QString &);
	void eventRecommenderName(const QString &);
	void eventWantsReminderEmail(bool dummy);
	void eventWeekly(bool on);
	void eventBiMonthly(bool on);
	void eventMonthly(bool on);
	void eventWantsRegularInformations(bool);
	void eventWantsToHelp(bool);
	void eventWantsToAnalyze(bool);
	void eventEmail(const QString& dummy);
	void eventMixdateYear(int dummy);
	void eventMixdateMonth(int dummy);
	void eventArrivaldateYear(int dummy);
	void eventArrivaldateMonth(int dummy);
	void eventMessage();
	void eventServiceSwitchToggled(bool);
	void eventOpinionSwitchToggled(bool);
	void eventOldSettingsButton();
	void eventSaveButton();
	void eventTabChanged(QWidget *);

signals:
	void Event(QString eventname);
	void QuitEvent(int);
	void ByeByeEnded();

private:
	ClientStorage * theData;
};
#endif // DIALOGS_H





