// usage.cpp
// observe the user behaviour
// v0.84 (7.3.2002)
// (c) 2002 Andreas Krueger
//
// Please, if you change anything in this code,
// mark the end of those lines with '// *+* your name *+* ' 
// (so that I can find it) and send the code 
// to cye-code@AndreasKrueger.de
//
// last change: 29.5.2002
//          by: AK

#include"usage.h"
#include"constants.h"
#include"helpers.h"

// UserEvent holds one event

QDataStream& operator<< (QDataStream &s, UserEvent UE){
	return s << UE.time << UE.theID;
}
QDataStream& operator>> (QDataStream &s, UserEvent &UE){
	return s >> UE.time >> UE.theID;
}
bool operator== (UserEvent &UE1, UserEvent &UE2){
	return ( UE1.theID == UE2.theID  && UE1.time == UE2.time );
}


// 
// User Observation
// 

#include<qtimer.h>

Usage::Usage(QWidget * parent, const char * name)
: QObject(parent, name), EventName2ID(mapEventName2ID), ID2EventName(mapID2EventName)
{
	initEventIDTextMapping();
}

void Usage::setDataTo(QValueList<UserEvent>& oneSession, QMap<Q_UINT16, Q_UINT32>& oneTotalEvents){
	Session=oneSession;
	TotalEvents=oneTotalEvents;
}
void Usage::setDataTo(QMap<Q_UINT16, Q_UINT32>& oneTotalEvents){
	TotalEvents=oneTotalEvents;
}
void Usage::setDataTo(QValueList<UserEvent>& oneSession){
	Session=oneSession;
}

void Usage::startObservation(const QMap<Q_UINT16, Q_UINT32> & OldTotalEvents){
	ObservationAllowed=true;
	TotalEvents.clear();
	TotalEvents=OldTotalEvents;
	Session.clear();
	timer.start();
	this->StoreEvent("Start");
	this->StoreEvent("TimerStarted");
	QTimer::singleShot( 85000*1000, this, SLOT(restartTimer()) );
	        // original plan: restart after 16 Mio hundredth-seconds (24 bit)
	        // 160,000 seconds = 44 hours
	        // N.B.: 160,000,000 << 2,147,483,647 = INT_MAX 
	        // but: Qt: "Note that the counter wraps to zero 
	        // 24 hours after the last call to start() or restart."
	        // 85000 seconds = 23:36:40 hours 
}

void Usage::restartTimer(){  // necessary to prevent unnoticed 24h wrapping
	this->StoreEvent("TimerRestart");
	QTimer::singleShot( 85000*1000, this, SLOT(restartTimer()) );
	        // original plan: restart after 16 Mio hundredth-seconds (24 bit)
	        // 160,000 seconds = 44 hours
	        // N.B.: 160,000,000 << 2,147,483,647 = INT_MAX 
	        // but:  QTime::elapsed(): "Note that the counter wraps to zero 
	        // 24 hours after the last call to start() or restart."
	        // noticed wrapping after 85,000 seconds = 23:36:40 hours 
	timer.restart();
	this->StoreEvent("TimerStarted");
}

void Usage::toggleObservationAllowed(bool status){
	if (ObservationAllowed==status)
		return;
	else{
		if (status==true){
			ObservationAllowed=status;
			this->StoreEvent("ObservationAllowedAgain");
		}
		else{
			Session.clear();
			this->StoreEvent("ObservationNotAllowedAnymore");
			ObservationAllowed=status;
		}
	}
}

bool Usage::StoreEvent(QString eventName){
	if (!ObservationAllowed)
		return false;
	else{
		if (EventName2ID.contains(eventName)){
			Q_UINT16 ID=EventName2ID[eventName];
				// QTime::elapsed()
				// Warning: If the system's clock setting has been changed 
				// since the last time start() or restart() was called, 
				// the result is undefined. This can happen 
				// e.g. when daylight saving is turned on or off. 
			Session.append(UserEvent(timer.elapsed(), ID));

			TotalEvents[ID] ++;

			return true;
		}
		else {
			qDebug(tr("ERROR Usage::StoreEvent : there is no event called %1").arg(eventName) );
			this->StoreEvent("unknown");
			return false;
		}
	}
}

void Usage::initEventIDTextMapping(){
	mapEventName2ID.clear();
	mapID2EventName.clear();

	// Create the mapping for the main program:
	uint i;
	for (i=0; i<numIDtexts;i++){ // lnge des arrays automatisch?
		mapEventName2ID.insert(IDtexts[i], i);
		mapID2EventName.insert(i, IDtexts[i]);
	}
	// Create the mapping for the settings dialog ( IDs > 200 )
	for (i=0; i<numSettingsIDtexts;i++){ // lnge des arrays automatisch?
		mapEventName2ID.insert(SettingsIDtexts[i], 200+i);
		mapID2EventName.insert(200+i, SettingsIDtexts[i]);
	}
	// Create the mapping for the client dialog ( IDs > 400 )
	for (i=0; i<numClientIDtexts;i++){ // lnge des arrays automatisch?
		mapEventName2ID.insert(ClientIDtexts[i], 400+i);
		mapID2EventName.insert(400+i, ClientIDtexts[i]);
	}
	// Create the mapping for the invitations dialog ( IDs > 600 )
	for (i=0; i<numInvitationIDtexts;i++){ // lnge des arrays automatisch?
		mapEventName2ID.insert(	InvitationIDtexts[i], 600+i);
		mapID2EventName.insert(600+i, 	InvitationIDtexts[i]);
	}

	// Create the mapping for the Events
	// cell-clicked(>1000), -edited(>2000), -focussed(>3000)
	QString eventname, wholeeventname, coinstring, countrystring;
	int index;
	for (i=0; i<numTableIDtexts ; i++){
		index=0;
		eventname=TableIDtexts[i];
		for (uint coin=0;coin<NumCoins;coin++){
			coinstring.setNum(coin);
			for (uint country=0;country<NumCountries;country++){
				countrystring.setNum(country);
				wholeeventname= eventname + "(" + coinstring + "," + countrystring + ")";
				mapEventName2ID.insert( wholeeventname, (i+1)*1000 + index);
				mapID2EventName.insert((i+1)*1000 + index, wholeeventname);
				index++;
			}
		}
	}
	mapEventName2ID.insert("unknown", 65535);
	mapID2EventName.insert(65535, "unknown");
}

#include <strstream>
using std::ostrstream;

char* Usage::giveAllEventsAsTextTupels(char* thirdcolumn){
	ostrstream ss;
	// Print the mapping for the main program:
	uint i;
	for (i=0; i<numIDtexts;i++){ // lnge des arrays automatisch?
		ss<<"("<<i<<",'"<<IDtexts[i].latin1()<<"'"<<thirdcolumn<<"),";
	}

	// Print the mapping for the settings dialog ( IDs > 200 )
	for (i=0; i<numSettingsIDtexts;i++){ // lnge des arrays automatisch?
		ss<<"("<<200+i<<",'"<<SettingsIDtexts[i].latin1()<<"'"<<thirdcolumn<<"),";
	}
	// Print the mapping for the client dialog ( IDs > 400 )
	for (i=0; i<numClientIDtexts;i++){ // lnge des arrays automatisch?
		ss<<"("<<400+i<<",'"<<ClientIDtexts[i].latin1()<<"'"<<thirdcolumn<<"),";
	}
	// Print the mapping for the invitations dialog ( IDs > 600 )
	for (i=0; i<numInvitationIDtexts;i++){ // lnge des arrays automatisch?
		ss<<"("<<600+i<<",'"<<InvitationIDtexts[i].latin1()<<"'"<<thirdcolumn<<"),";
	}

	// Print the mapping for the Events
	// cell-clicked(>1000), -edited(>2000), -focussed(>3000)
	QString eventname, wholeeventname, coinstring, countrystring;
	int index;
	for (i=0; i<numTableIDtexts ; i++){
		index=0;
		eventname=TableIDtexts[i];
		for (uint coin=0;coin<NumCoins;coin++){
			coinstring.setNum(coin);
			for (uint country=0;country<NumCountries;country++){
				countrystring.setNum(country);
				wholeeventname= eventname + "(" + coinstring + "," + countrystring + ")";
				ss<<"("<<(i+1)*1000 + index<<",'"<<wholeeventname.latin1()<<"'"<<thirdcolumn<<"),";
				index++;
			}
		}
	}

	ss<<"("<<65535<<",'"<<"unknown"<<"'"<<thirdcolumn<<"),";

	ss.seekp(ss.tellp()-(long)1); // cut out the last comma
	ss<<'\0';

	return ss.str();
}


void Usage::PrintEventTo(QTextOStream & s, QValueList<UserEvent>::Iterator UE)
{
	s <<"at "<<(*UE).TimeSecs()<<" seconds: ("<<(*UE).ID()<<") "<<ID2EventName[(*UE).ID()];
}

void Usage::PrintSessionTo(QTextOStream &s){
	QValueList<UserEvent>::Iterator event=Session.begin();
	for (; event!=Session.end();event++){
		PrintEventTo(s, event);
		s <<"\n";
	}
}

void Usage::PrintTotalEventsTo(QTextOStream &s){
	QMap<Q_UINT16,Q_UINT32>::Iterator event=TotalEvents.begin();
	for (; event!=TotalEvents.end();event++){
		s << ID2EventName[event.key()] << ":"<<event.data() <<"\n";
	}
}

void Usage::PrepareSessionListView(QListView * LV){

	LV->clear();
    LV->setSorting( -1 ); // no sorting
    LV->addColumn( "Time in s" );
    LV->addColumn( "Event ID" );
	LV->addColumn( "Event name" );

	QString time, ID, name;
	QValueList<UserEvent>::Iterator event=Session.begin();
	for (; event!=Session.end();event++){
		time.setNum((*event).TimeSecs());
		ID.setNum((*event).ID());
		name=ID2EventName[(*event).ID()];
		LV->insertItem(new QListViewItem(LV, time, ID, name));
	}
}

void Usage::UpdateSessionListView(QListView * LV){
	LV->clear();
	QString time, ID, name;
	QListViewItem *item=0;
	QValueList<UserEvent>::Iterator event=Session.begin();
	for (; event!=Session.end();event++){
		time.setNum((*event).TimeSecs());
		ID.setNum((*event).ID());
		name=ID2EventName[(*event).ID()];
//		LV->insertItem(new QListViewItem(LV, time, ID, name));
		item=new QListViewItem(LV, time, ID, name);
	}
	LV->ensureItemVisible ( item );
}

void Usage::PrepareAllEventsListView(QListView * LV){

	LV->clear();
    LV->setSorting( -1 ); // no sorting
	LV->addColumn( "Event ID" );
	LV->addColumn( "Counter" );
	LV->addColumn( "Event Name" );
	UpdateAllEventsListView(LV);

}

void Usage::UpdateAllEventsListView(QListView * LV){
	LV->clear();
	QString counter, ID, name;
	QMap<Q_UINT16, Q_UINT32>::Iterator event=TotalEvents.begin();
	for (; event!=TotalEvents.end();event++){
		counter.setNum(event.data());
		ID.setNum(event.key());
		name=ID2EventName[event.key()];
		LV->insertItem(new QListViewItem(LV, ID, counter, name));
	}
}
Q_UINT32 Usage::AllEventsCreateListViewItemChildrenOf(QListViewItem * LV){
	QString counter, ID, name;
	Q_UINT32 NumTotalEvents=0;
	QListViewItem *first=0, *actual=0;
	QMap<Q_UINT16, Q_UINT32>::Iterator event=TotalEvents.begin();
	for (; event!=TotalEvents.end();event++){
		counter.setNum(event.data());
		NumTotalEvents+=event.data();
		ID.setNum(event.key());
		name=ID2EventName[event.key()];
		actual=new QListViewItem(LV, name, counter, ID);
		if (event==TotalEvents.begin()) 
			first=actual;
	}
	invertOrderOf(LV,first);
	return NumTotalEvents;
}

Q_UINT32 Usage::SessionEventsCreateListViewItemChildrenOf(QListViewItem * LV){
	QString time, ID, name;
	QListViewItem *item;
	QValueList<UserEvent>::Iterator event=Session.begin();
	float TimeSecs;
	for (; event!=Session.end();event++){
		TimeSecs=(*event).TimeSecs();
		time="%1 secs";time=time.arg(TimeSecs);
		ID.setNum((*event).ID());
		name=ID2EventName[(*event).ID()];
		item=new QListViewItem(LV, time, name,ID);
	}
	return Session.count();
}


void Usage::testMapping(){
	// just testing:
	QString test, IDnum, IDnum2;
	uint size=mapID2EventName.count();
	uint size2=mapEventName2ID.count();
	for (uint i=0;i<=65535;i++){
		test=ID2EventName[i];		// forwards test
		IDnum.setNum(i);
		IDnum2.setNum(EventName2ID[test]); // backwards test
		if (!test.isEmpty()) 
			qDebug(IDnum +" : "+ test +" : "+IDnum2);
		if (size!=ID2EventName.count() || size!=ID2EventName.count()
			|| size2!=EventName2ID.count() || size2!= EventName2ID.count()){
			qDebug(tr("map-access changed size of maps %1 %2").arg(size).arg(size2));
		}
	}
}


void Usage::testSession(){
	this->StoreEvent("Quit");
	this->StoreEvent("Quitten");
	QString str; QTextOStream tos(&str) ;
	PrintSessionTo( tos );
	qDebug("The whole session looks like this:");
	qDebug(str);
}

